@@ -10,7 +10,7 @@ import warnings
 from base64 import b64decode
 from configparser import ConfigParser
 from copy import copy, deepcopy
-from email import message_from_bytes, message_from_string
+from email import message_from_bytes, message_from_string, header
 from email.message import EmailMessage, Message
 from email.parser import BytesParser
 from email.policy import default as policy
@@ -138,26 +138,23 @@ class Envelope:
         result = []
         for a in self._attachments:  # include attachments info as they are removed with the payload later
             if a.inline:
-                s = f"Inline attachment {a.name} ({a.mimetype}): <img src='cid:{a.inline}'/>"
+                s = f"Inline attachment {a.preview()}"
             else:
-                s = f"Attachment {a.name} ({a.mimetype}): {a.get_sample()}"
+                s = f"Attachment {a.preview()}"
             result.append(s)
 
         if self._bcc:  # as bcc is not included as an e-mail header, we explicitly states it here
             result.append("Bcc: " + ", ".join(map(str, self._bcc)))
 
-        for i, r in enumerate(self._result):
+        for r in self._result:
             if isinstance(r, Message):  # smime library always produces a Message object, otherwise EmailMessage is got
                 if self._sign or self._encrypt:
                     result.append(("GPG" if self._gpg else "S/MIME") + ": " + ", ".join(
                         x for x in [bool(self._sign) and "signed", bool(self._encrypt) and "encrypted"] if x))
 
-                # remove body from the message because it may have become unreadable, we will be calling self.message() instead
-                # I cannot use r.set_payload(""), since it would change the headers like Content-Transfer-Encoding
-                for line in r.as_string().splitlines():
-                    result.append(line)
-                    if not line.strip():
-                        break
+                # append headers
+                [result.append(f"{key}: {val}") for key, val in r.items()]
+                result.append("")
             else:
                 result.append(r)
 
@@ -209,8 +206,8 @@ class Envelope:
         e = Envelope()
         try:
             return Parser(e, key=key, cert=cert, gnupg_home=e._get_gnupg_home()).parse(o, add_headers=True)
-        except ValueError as e:
-            logger.warning(f"Message might not have been loaded correctly. {e}")
+        except ValueError as err:
+            logger.warning(f"Message might not have been loaded correctly. {err}")
 
         # emergency body loading when parsing failed
         header_row = re.compile(r"([^\t:]+):(.*)")
@@ -330,9 +327,6 @@ class Envelope:
         # if a parameter is not set, use class defaults, else init with parameter
         self._populate(locals())
 
-        if sign or encrypt or send is not None:
-            self._start(send=send)
-
     @staticmethod
     def _get_private_var(k):
         """ Gets internal specific interface var name from its method name. """
@@ -342,7 +336,7 @@ class Envelope:
 
     def _populate(self, params):
         for k, v in params.items():
-            if k in ["self", "send"]:
+            if k in ["self", "send"]:  # send must be the last
                 continue
             elif k == "smime":  # smime uses _gpg, not _smime because it needs no parameter
                 if v is True:
@@ -377,6 +371,8 @@ class Envelope:
                 getattr(self, k)(v)  # ex: self.message(message)
 
         self._prepare_from()
+        if params.get("sign") or params.get("encrypt") or params.get("send") is not None:
+            self._start(send=params.get("send"))
         return self
 
     def copy(self):
@@ -385,54 +381,40 @@ class Envelope:
 
     @staticmethod
     def _parse_addresses(registry, email_or_list):
-        registry += (a for a in Address.parse(email_or_list) if a not in registry)
+        addresses = assure_list(email_or_list)
+        if any(not x for x in addresses):
+            registry.clear()
+        addresses = [x for x in addresses if x]  # filter out possible "" or False
+        if addresses:
+            registry += (a for a in Address.parse(addresses) if a not in registry)
 
-    def to(self, email_or_list=None, name=None, address=None) -> Union["Envelope", List[str], List[Address]]:
+    def to(self, email_or_list=None) -> Union["Envelope", List[Address]]:
         """ Multiple addresses may be given in a string, delimited by comma (or semicolon).
          (The same is valid for `to`, `cc`, `bcc` and `reply-to`.)
 
-            e = (Envelope()
-                .to("person1@example.com")
-                .to("person1@example.com, John <person2@example.com>")
-                .to(["person3@example.com"]))
-
-            e.to()  # ["person1@example.com", "John <person2@example.com>", "person3@example.com"] (Address objects)
-            e.to(address=False)  # ["", "John", ""]
-            e.to(name=True)  # ["person1@example.com", "John", "person3@example.com"] # returns address if no name given
-            e.to(address=True)  # ["person1@example.com", "person2@example.com", "person3@example.com"]  # address only
-
-            XXX `name` and `address` are undocumented and untested features yet. How would they work?
-                Should we put these parameters to the other .cc, .bcc functions, or should we return Address objects directly?
-                return: Envelope if `email_or_list` set or List[Address] if neither `address` nor `name` set or List[str].
-                Address objects are equal if their e-mail address are equal.
-                But that way, we have to manual convert to str while joining ",".join(map(str, self.to())).
-                What is the default user use case?
-                -> I suppose WE RETURN OBJECTS.
-
             :param email_or_list: str|List[str] Set e-mail address/es. If None, we are reading.
-            :param address: bool Include e-mail addresses in the reading result.
-            :param name: bool Include e-mail names in the reading result.
+            return: Envelope if `email_or_list` set or List[Address]
         """
         if email_or_list is None:
-            return [x.get(name, address) for x in self._to]
+            return self._to
         self._parse_addresses(self._to, email_or_list)
         return self
 
-    def cc(self, email_or_list=None) -> Union["Envelope", List[str]]:
+    def cc(self, email_or_list=None) -> Union["Envelope", List[Address]]:
         if email_or_list is None:
-            return [str(x) for x in self._cc]
+            return self._cc
         self._parse_addresses(self._cc, email_or_list)
         return self
 
-    def bcc(self, email_or_list=None) -> Union["Envelope", List[str]]:
+    def bcc(self, email_or_list=None) -> Union["Envelope", List[Address]]:
         if email_or_list is None:
-            return [str(x) for x in self._bcc]
+            return self._bcc
         self._parse_addresses(self._bcc, email_or_list)
         return self
 
-    def reply_to(self, email_or_list=None) -> Union["Envelope", List[str]]:
+    def reply_to(self, email_or_list=None) -> Union["Envelope", List[Address]]:
         if email_or_list is None:
-            return [str(x) for x in self._reply_to]
+            return self._reply_to
         self._parse_addresses(self._reply_to, email_or_list)
         return self
 
@@ -532,17 +514,17 @@ class Envelope:
             self.header("Date", date)
         return self
 
-    def sender(self, email=None) -> Union["Envelope", str]:
+    def sender(self, email=None) -> Union["Envelope", Address]:
         """  Alias for "from" if not set. Otherwise appends header "Sender". If None, current `Sender` returned. """
         if email is None:
-            return str(self.__sender or "")
+            return self.__sender
         self._sender = Address.parse(email, single=True, allow_false=True)
         self._prepare_from()
         return self
 
-    def from_(self, email=None) -> Union["Envelope", str]:
+    def from_(self, email=None) -> Union["Envelope", Address]:
         if email is None:
-            return str(self.__from or "")
+            return self.__from
         self._from = Address.parse(email, single=True, allow_false=True)
         self._prepare_from()
         return self
@@ -640,7 +622,7 @@ class Envelope:
         """
 
         # lowercase header to its method name
-        specific_interface = {"to": self.to, "cc": self.cc, "bcc": self.bcc, "reply_to": self.reply_to,
+        specific_interface = {"to": self.to, "cc": self.cc, "bcc": self.bcc, "reply-to": self.reply_to,
                               "from": self.from_, "sender": self.sender,
                               "subject": self.subject
                               }
@@ -1298,6 +1280,7 @@ class Envelope:
                 # rather than plain text. Which could cause a transferring SMTP server to include line breaks and spaces
                 # that might break up DKIM.
                 msg_text.set_content(t.encode("utf-8"), maintype="text", subtype=mime)  # text as bytes
+                msg_text.set_param("charset", "utf-8", replace=True)
             else:
                 msg_text.set_content(t, subtype=mime)  # text as string
         else:
@@ -1316,7 +1299,13 @@ class Envelope:
                     # passing bytes to EmailMessage makes its ContentManager to transfer it via base64 or quoted-printable
                     # rather than plain text. Which could cause a transferring SMTP server to include line breaks and spaces
                     # that might break up DKIM.
-                    msg_text.add_alternative(html.encode("utf-8"), maintype="text", subtype='html')  # `html` as bytes
+                    
+                    # create an alternative message part and set utf-8 encoding explicitly
+                    alt_msg = EmailMessage()
+                    alt_msg.set_content(html.encode("utf-8"), maintype="text", subtype="html")  # `html` as bytes
+                    alt_msg.set_param("charset", "utf-8", replace=True)
+                    msg_text.make_alternative()
+                    msg_text.attach(alt_msg)
                 else:
                     msg_text.add_alternative(html, subtype='html')  # `html` as string
             except (ValueError, TypeError):
@@ -1335,7 +1324,7 @@ class Envelope:
             o.make_related()
             [o.add_related(a.data,
                            **dict(zip(("maintype", "subtype"), a.mimetype.split("/"))),
-                           cid=a.name) for a in self._attachments if a.inline]
+                           cid=f"<{a.name}>") for a in self._attachments if a.inline]
 
         if self._attach_key:
             # send your public key as an attachment (so that it can be imported before it propagates on the server)
@@ -1378,7 +1367,7 @@ class Envelope:
                 email["Subject"] = self._subject
         return email
 
-    def recipients(self, *, clear=False) -> Set[str]:
+    def recipients(self, *, clear=False) -> Set[Address]:
         """ Return set of all recipients – To, Cc, Bcc
             :param: clear If true, all To, Cc and Bcc recipients are removed and the object is returned.
 
@@ -1393,7 +1382,7 @@ class Envelope:
             self._cc.clear()
             self._bcc.clear()
             return self
-        return {str(x) for x in set(self._to + self._cc + self._bcc)}
+        return {x for x in set(self._to + self._cc + self._bcc)}
 
     def attachments(self, name=None, inline=None) -> Union[Attachment, List[Attachment]]:
         """ Access the attachments.
@@ -1497,6 +1486,9 @@ class Parser:
                 # We skip MIME-Version because it may be another one in a encrypted sub-message we take the headers from too.
                 if k in ("Content-Type", "Content-Transfer-Encoding", "MIME-Version"):
                     continue
+                if isinstance(val, header.Header):
+                    # when diacritics appear in Subject, object is returned instead of a string
+                    val = val.encode()
                 self.e.header(k, " ".join(x.strip() for x in val.splitlines()))
         maintype, subtype = o.get_content_type().split("/")
         if o.is_multipart():
@@ -1531,7 +1523,10 @@ class Parser:
                 raise ValueError(f"Subtype {subtype} not implemented")
         elif maintype == "text":
             if subtype in (HTML, PLAIN):
-                self.e.message(o.get_payload(decode=True).strip(), alternative=subtype)
+                t = o.get_payload(decode=True).strip()
+                if o.get_charsets() and o.get_charsets()[0]:
+                    t = t.decode(o.get_charsets()[0])
+                self.e.message(t, alternative=subtype)
             else:
                 raise ValueError(f"Unknown subtype: {subtype}")
         elif maintype == "application" and subtype == "x-pkcs7-mime":  # decrypting S/MIME
